import { __rest, __assign } from './../external/tslib/tslib.es6.js';
import React, { forwardRef, useState, useRef, useEffect } from 'react';
import classnames from 'classnames';
import toPairs from 'lodash/toPairs';
import groupBy from 'lodash/groupBy';
import { Typography } from '@highlight-ui/typography';
import { parseMetadata, suffixMetadata } from '@highlight-ui/utils-commons';
import styles from './List.module.scss.js';
import Option from './Option.js';
import OptionGroup from './OptionGroup.js';
import SearchInput from './SearchInput.js';
import { searchSubstring } from './utils.js';
import AddOptionsFooter from './AddOptionsFooter.js';
import LoadingState from './LoadingState.js';
import SubmitFooter from './SubmitFooter.js';
import SelectAllOption from './SelectAllOption.js';
var MIN_OPTIONS_TO_DISPLAY_SELECT_ALL = 7.5;
var MIN_GROUPS_TO_DISPLAY_HEADER = 2;

function List(_a, ref) {
  var _b;

  var className = _a.className,
      selectAllLabel = _a.selectAllLabel,
      highlightedText = _a.highlightedText,
      variant = _a.variant,
      multiple = _a.multiple,
      onSelect = _a.onSelect,
      onSelectAll = _a.onSelectAll,
      onSearchChange = _a.onSearchChange,
      options = _a.options,
      optionType = _a.optionType,
      emptyStateLabel = _a.emptyStateLabel,
      renderEmptyState = _a.renderEmptyState,
      searchable = _a.searchable,
      searchInitialValue = _a.searchInitialValue,
      searchAutofocus = _a.searchAutofocus,
      searchPlaceholder = _a.searchPlaceholder,
      searchNoResultsLabel = _a.searchNoResultsLabel,
      minOptionsToDisplaySearch = _a.minOptionsToDisplaySearch,
      selectedOptionsWithValuesOnly = _a.selectedOptionsWithValuesOnly,
      creatableOptions = _a.creatableOptions,
      style = _a.style,
      isLoading = _a.isLoading,
      submitOptions = _a.submitOptions,
      onSubmit = _a.onSubmit,
      onCancel = _a.onCancel,
      metadata = _a.metadata,
      restProps = __rest(_a, ["className", "selectAllLabel", "highlightedText", "variant", "multiple", "onSelect", "onSelectAll", "onSearchChange", "options", "optionType", "emptyStateLabel", "renderEmptyState", "searchable", "searchInitialValue", "searchAutofocus", "searchPlaceholder", "searchNoResultsLabel", "minOptionsToDisplaySearch", "selectedOptionsWithValuesOnly", "creatableOptions", "style", "isLoading", "submitOptions", "onSubmit", "onCancel", "metadata"]);

  var _c = useState(searchInitialValue),
      searchInputValue = _c[0],
      setSearchInputValue = _c[1];

  var searchInputRef = useRef(null);
  useEffect(function () {
    if (!searchAutofocus || !searchInputRef.current) return;
    setTimeout(function () {
      if (searchInputRef.current) searchInputRef.current.focus();
    }); // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  var shouldShowSelectAll = multiple && options.length > MIN_OPTIONS_TO_DISPLAY_SELECT_ALL && searchInputValue.length === 0;
  var selectAllIsSelected = options.length === selectedOptionsWithValuesOnly.length;
  var filteredOptions = options.filter(function (option) {
    return searchSubstring(option.label, searchInputValue) !== -1;
  });
  var shouldShowNoResultsLabel = searchInputValue !== '' && filteredOptions.length === 0;
  var shouldShowSearchInput = searchable && options.length >= minOptionsToDisplaySearch;
  var groupedOptions = groupBy(filteredOptions, function (option) {
    return option.group;
  });
  var shouldShowGroups = Object.keys(groupedOptions).length >= MIN_GROUPS_TO_DISPLAY_HEADER;

  var renderOptions = function (optionsList) {
    var optionsRendered = optionsList.map(function (option) {
      return /*#__PURE__*/React.createElement(Option, {
        disabled: option.disabled,
        highlightedText: searchable ? searchInputValue : highlightedText,
        key: option.value,
        onSelect: function () {
          return onSelect(option);
        },
        option: option,
        optionStyle: multiple ? 'checkbox' : 'checkmark',
        optionType: optionType,
        selected: selectedOptionsWithValuesOnly.includes(option.value)
      });
    });
    return /*#__PURE__*/React.createElement(React.Fragment, null, shouldShowSelectAll && !shouldShowGroups && /*#__PURE__*/React.createElement(SelectAllOption, {
      onSelect: onSelectAll,
      isSelected: selectAllIsSelected,
      label: selectAllLabel
    }), optionsRendered);
  };

  var renderOptionGroups = function () {
    var groupsRendered = toPairs(groupedOptions).map(function (_a) {
      var groupName = _a[0],
          groupOptions = _a[1];
      return /*#__PURE__*/React.createElement(OptionGroup, {
        key: groupName,
        name: groupName
      }, renderOptions(groupOptions));
    });
    return /*#__PURE__*/React.createElement(React.Fragment, null, shouldShowSelectAll && /*#__PURE__*/React.createElement(SelectAllOption, {
      onSelect: onSelectAll,
      isSelected: selectAllIsSelected,
      label: selectAllLabel
    }), groupsRendered);
  };

  var handleSearchInputChange = function (search) {
    setSearchInputValue(search);
    onSearchChange(search);
  };

  return /*#__PURE__*/React.createElement("div", __assign({
    ref: ref,
    className: classnames(styles.list, className, (_b = {}, _b[styles.multiLineList] = optionType === 'multi-line', _b[styles.fullWidth] = variant === 'full-width', _b)),
    style: style
  }, parseMetadata(suffixMetadata(metadata, 'list')), restProps), isLoading ? /*#__PURE__*/React.createElement(LoadingState, null) : /*#__PURE__*/React.createElement(React.Fragment, null, shouldShowSearchInput && /*#__PURE__*/React.createElement(SearchInput, {
    ref: searchInputRef,
    placeholder: searchPlaceholder,
    value: searchInputValue,
    onTextChange: handleSearchInputChange
  }), shouldShowNoResultsLabel && /*#__PURE__*/React.createElement(Typography, {
    component: "div",
    variant: "body-1",
    className: styles.noResults
  }, searchNoResultsLabel + " \"" + searchInputValue + "\""), !shouldShowNoResultsLabel && (options.length === 0 ? renderEmptyState({
    label: emptyStateLabel
  }) : /*#__PURE__*/React.createElement("ul", {
    role: "listbox"
  }, shouldShowGroups ? renderOptionGroups() : renderOptions(filteredOptions))), creatableOptions && /*#__PURE__*/React.createElement(AddOptionsFooter, {
    onCreate: creatableOptions.onCreate,
    createButtonProps: creatableOptions.createButtonProps,
    inputProps: creatableOptions.inputProps,
    colorPickerOptions: creatableOptions.colorPickerOptions
  }), !creatableOptions && submitOptions && /*#__PURE__*/React.createElement(SubmitFooter, {
    onSubmit: onSubmit,
    onCancel: onCancel,
    submitLabel: submitOptions.submitLabel,
    cancelSubmitLabel: submitOptions.cancelLabel
  })));
}

var ListWithRef = /*#__PURE__*/forwardRef(List);
export { ListWithRef as default };
//# sourceMappingURL=List.js.map
