import { __assign } from './../external/tslib/tslib.es6.js';
import React, { useState, useEffect, useMemo, useRef } from 'react';
import './../external/react-click-outside-hook/build/index.js';
import noop from 'lodash/noop';
import { useAutoPositioner, useForkRef } from '@highlight-ui/utils-hooks';
import { parseMetadata } from '@highlight-ui/utils-commons';
import { getFixedOptions, getAllSelectableOptions, getSelectedValues } from './helpers.js';
import ListWithRef from './List.js';
import DefaultEmptyState from './DefaultEmptyState.js';
import DefaultTrigger from './DefaultTrigger.js';
import { b as build } from '../_virtual/index.js';

var useListVisibilityChangeHandler = function (onListVisibilityChange, visible) {
  var initialRender = useRef(true);
  useEffect(function () {
    if (initialRender.current) {
      initialRender.current = false;
    } else {
      onListVisibilityChange(visible);
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [visible]);
};

function SelectWithRef(_a, ref) {
  var className = _a.className,
      listClassName = _a.listClassName,
      _b = _a.closeOnSelect,
      closeOnSelect = _b === void 0 ? false : _b,
      _c = _a.disabled,
      disabled = _c === void 0 ? false : _c,
      _d = _a.variant,
      variant = _d === void 0 ? 'inline' : _d,
      _e = _a.selectAllLabel,
      selectAllLabel = _e === void 0 ? 'Select all' : _e,
      metadata = _a.metadata,
      _f = _a.multiple,
      multiple = _f === void 0 ? false : _f,
      _g = _a.onSearchChange,
      onSearchChange = _g === void 0 ? noop : _g,
      _h = _a.onSelect,
      onSelect = _h === void 0 ? noop : _h,
      _j = _a.onListVisibilityChange,
      onListVisibilityChange = _j === void 0 ? noop : _j,
      options = _a.options,
      _k = _a.emptyStateLabel,
      emptyStateLabel = _k === void 0 ? 'Nothing to display' : _k,
      _l = _a.renderEmptyState,
      renderEmptyState = _l === void 0 ? DefaultEmptyState : _l,
      _m = _a.renderTrigger,
      renderTrigger = _m === void 0 ? DefaultTrigger : _m,
      _o = _a.optionType,
      optionType = _o === void 0 ? 'single-line' : _o,
      _p = _a.searchable,
      searchable = _p === void 0 ? true : _p,
      _q = _a.searchAutofocus,
      searchAutofocus = _q === void 0 ? true : _q,
      _r = _a.searchInitialValue,
      searchInitialValue = _r === void 0 ? '' : _r,
      _s = _a.searchNoResultsLabel,
      searchNoResultsLabel = _s === void 0 ? 'No results for' : _s,
      _t = _a.searchPlaceholder,
      searchPlaceholder = _t === void 0 ? 'Search' : _t,
      _u = _a.selectedOptions,
      selectedOptions = _u === void 0 ? [] : _u,
      _v = _a.minOptionsToDisplaySearch,
      minOptionsToDisplaySearch = _v === void 0 ? 8 : _v,
      _w = _a.triggerLabel,
      triggerLabel = _w === void 0 ? '' : _w,
      _x = _a.outline,
      outline = _x === void 0 ? 'default' : _x,
      _y = _a.autoFocus,
      autoFocus = _y === void 0 ? false : _y,
      triggerId = _a.triggerId,
      creatableOptions = _a.creatableOptions,
      _z = _a.isClearable,
      isClearable = _z === void 0 ? false : _z,
      _0 = _a.highlightedText,
      highlightedText = _0 === void 0 ? '' : _0,
      _1 = _a.isLoading,
      isLoading = _1 === void 0 ? false : _1,
      submitOptions = _a.submitOptions;

  var _2 = build.exports.useClickOutside(),
      clickOutsideRef = _2[0],
      hasClickedOutside = _2[1];

  var _3 = useState(false),
      visible = _3[0],
      setVisibility = _3[1];

  var _4 = useAutoPositioner(),
      setReferenceElement = _4.setReferenceElement,
      setPopperElement = _4.setPopperElement,
      popperStyles = _4.styles.popper,
      attributes = _4.attributes;

  useListVisibilityChangeHandler(onListVisibilityChange, visible);
  useEffect(function () {
    if (hasClickedOutside && visible) {
      setVisibility(false);

      if (submitOptions) {
        submitOptions.onCancel();
      }
    }
  }, [hasClickedOutside, visible, submitOptions]);
  var selectedOptionsWithValuesOnly = useMemo(function () {
    return selectedOptions.map(function (selectedOption) {
      return selectedOption.value;
    });
  }, [selectedOptions]);

  var beforeSelect = function (callback) {
    return function () {
      var args = [];

      for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
      }

      if (closeOnSelect) setVisibility(false);
      callback.apply(void 0, args);
    };
  };

  var handleClear = function (evt) {
    evt.stopPropagation();
    var fixedOptions = getFixedOptions(options, selectedOptions);
    onSelect(fixedOptions);

    if (submitOptions) {
      submitOptions.onSubmit(fixedOptions);
    }

    setVisibility(false);
  };

  var handleSelectAll = function (selected) {
    var opts = selected ? getAllSelectableOptions(options, selectedOptions) : getFixedOptions(options, selectedOptions);
    onSelect(opts);
  };

  var handleSelect = function (option) {
    onSelect(getSelectedValues({
      option: option,
      selectedOptions: selectedOptions,
      selectedOptionsWithValuesOnly: selectedOptionsWithValuesOnly,
      multiple: multiple
    }));
  };

  var handleTriggerClick = function () {
    setVisibility(function (currentVisibility) {
      return !currentVisibility;
    });

    if (visible && submitOptions) {
      submitOptions.onCancel();
    }
  };

  var finalRef = useForkRef([clickOutsideRef, ref]);

  var _5 = useState(),
      triggerWidth = _5[0],
      setTriggerWidth = _5[1];

  var triggerRef = useForkRef([setReferenceElement, function (elem) {
    if (elem) setTriggerWidth(getComputedStyle(elem).width);
  }]);

  var handleSubmit = function () {
    if (submitOptions) submitOptions.onSubmit(selectedOptions);
    setVisibility(false);
  };

  var handleCancelSubmit = function () {
    setVisibility(false);
    if (submitOptions) submitOptions.onCancel();
  };

  return /*#__PURE__*/React.createElement("div", __assign({
    className: className,
    ref: finalRef
  }, parseMetadata(metadata)), /*#__PURE__*/React.createElement("div", null, renderTrigger({
    selectRef: clickOutsideRef,
    ref: triggerRef,
    disabled: disabled,
    variant: variant,
    label: triggerLabel,
    multiple: multiple,
    onClick: handleTriggerClick,
    selectedOptions: selectedOptions,
    visible: visible,
    outline: outline,
    autoFocus: autoFocus,
    triggerId: triggerId,
    setListVisibility: setVisibility,
    onClear: isClearable ? handleClear : undefined
  })), visible && /*#__PURE__*/React.createElement(ListWithRef, __assign({
    className: listClassName,
    metadata: metadata,
    selectAllLabel: selectAllLabel,
    isLoading: isLoading,
    highlightedText: highlightedText,
    variant: variant,
    multiple: multiple,
    onSelect: beforeSelect(handleSelect),
    onSelectAll: beforeSelect(handleSelectAll),
    onSearchChange: onSearchChange,
    options: options,
    optionType: optionType,
    emptyStateLabel: emptyStateLabel,
    renderEmptyState: renderEmptyState,
    selectedOptionsWithValuesOnly: selectedOptionsWithValuesOnly,
    searchable: searchable,
    searchNoResultsLabel: searchNoResultsLabel,
    searchPlaceholder: searchPlaceholder,
    searchInitialValue: searchInitialValue,
    searchAutofocus: searchAutofocus,
    minOptionsToDisplaySearch: minOptionsToDisplaySearch,
    onSubmit: handleSubmit,
    onCancel: handleCancelSubmit,
    submitOptions: submitOptions,
    creatableOptions: creatableOptions ? __assign(__assign({}, creatableOptions), {
      onCreate: function (createdItem) {
        if (closeOnSelect && !multiple) setVisibility(false);
        creatableOptions.onCreate(createdItem);
      }
    }) : creatableOptions,
    ref: setPopperElement,
    style: __assign(__assign(__assign({}, popperStyles), variant === 'full-width' ? {
      width: triggerWidth
    } : {}), isLoading ? {
      minWidth: triggerWidth
    } : {})
  }, attributes.popper)));
}

SelectWithRef.displayName = 'Select';
/**
 * According to this article: https://fettblog.eu/typescript-react-generic-forward-refs/
 */

var Select = /*#__PURE__*/React.forwardRef(SelectWithRef);
export { Select, Select as default };
//# sourceMappingURL=Select.js.map
